@@ -8,17 +8,18 @@
 #define KOSMINDOORMAP_MAPCSSRESULT_P_H
 
 #include "mapcssdeclaration_p.h"
+#include "mapcsstypes.h"
 
 #include <vector>
 
 namespace KOSMIndoorMap {
 
-/** Result of MapCSS stylesheet evaluation. */
-class MapCSSResult
+/** Result of MapCSS stylesheet evaluation for a single layer selector. */
+class MapCSSResultItem
 {
 public:
-    explicit MapCSSResult();
-    ~MapCSSResult();
+    explicit MapCSSResultItem();
+    ~MapCSSResultItem();
 
     void clear();
 
@@ -34,17 +35,43 @@ public:
     /** The active declarations for the queried element. */
     const std::vector<const MapCSSDeclaration*>& declarations() const;
 
+    /** The layer selector for this result. */
+    LayerSelectorKey layerSelector() const;
+
     /** @internal */
     void addDeclaration(const MapCSSDeclaration *decl);
-    void addClass(const QByteArray &cls);
-    bool hasClass(const QByteArray &cls) const;
+    void addClass(ClassSelectorKey cls);
+    bool hasClass(ClassSelectorKey cls) const;
+    void setLayerSelector(LayerSelectorKey layer);
 
 private:
     std::vector<const MapCSSDeclaration*> m_declarations;
-    std::vector<QByteArray> m_classes;
+    std::vector<ClassSelectorKey> m_classes;
+    LayerSelectorKey m_layer;
     int m_flags = 0;
 };
 
+/** Result of MapCSS stylesheet evaluation for all layer selectors. */
+class MapCSSResult
+{
+public:
+    explicit MapCSSResult();
+    ~MapCSSResult();
+
+    void clear();
+
+    /** Results for all layer selectors. */
+    const std::vector<MapCSSResultItem>& results() const;
+
+    /** @internal */
+    MapCSSResultItem& operator[](LayerSelectorKey layer);
+    const MapCSSResultItem& operator[](LayerSelectorKey layer) const;
+
+private:
+    std::vector<MapCSSResultItem> m_results;
+    mutable std::vector<MapCSSResultItem> m_inactivePool; // for reuse of already allocated result items
+};
+
 }
 
 #endif // KOSMINDOORMAP_MAPCSSRESULT_P_H
